/* DPDK */
#define MEMPOOL_CACHE_SIZE 256
#define RTE_TEST_RX_DESC_DEFAULT 1024
#define RTE_TEST_TX_DESC_DEFAULT 1024
static uint16_t nb_rxd = RTE_TEST_RX_DESC_DEFAULT;
static uint16_t nb_txd = RTE_TEST_TX_DESC_DEFAULT;
struct rte_mempool *tx_mbuf_pool;
struct rte_mempool *rx_mbuf_pool;
struct rte_ether_addr s_addr;
struct rte_ether_addr d_addr;
static const struct rte_eth_conf port_conf_default = {
  .rxmode = { .max_lro_pkt_size = RTE_ETHER_MAX_LEN }
};
static inline int port_init(int portid, struct rte_mempool *rx_mbuf_pool) {
  struct rte_eth_conf port_conf = port_conf_default;
  const uint16_t rx_rings = 1, tx_rings = 1;
  int retval;
  uint16_t q;

  retval = rte_eth_dev_configure(portid, rx_rings, tx_rings, &port_conf);
  if (retval != 0)
    return retval;

  /* Allocate and set up 1 RX queue per Ethernet port. */
  for (q = 0; q < rx_rings; q++) {
    retval = rte_eth_rx_queue_setup(portid, q, nb_rxd,
        rte_eth_dev_socket_id(portid), NULL, rx_mbuf_pool);
    if (retval < 0)
      return retval;
  }

  /* Allocate and set up 1 TX queue per Ethernet port. */
  for (q = 0; q < tx_rings; q++) {
    retval = rte_eth_tx_queue_setup(portid, q, nb_txd,
        rte_eth_dev_socket_id(portid), NULL);
    if (retval < 0)
      return retval;
  }

  /* Start the Ethernet port. */
  retval = rte_eth_dev_start(portid);
  if (retval < 0)
    return retval;

  return 0;
}
static void init_dpdk(int argc, char ** argv) {
  unsigned int nb_mbufs;
  int ret;

  ret = rte_eal_init(argc, argv);
  if (ret < 0)
    rte_exit(EXIT_FAILURE, "initlize fail!");

  argc -= ret;
  argv += ret;

  nb_mbufs = RTE_MAX((nb_rxd + nb_txd + BURST_SIZE + MEMPOOL_CACHE_SIZE), 8192U);
  nb_mbufs = 1024U * 16 - 1;

  tx_mbuf_pool = rte_pktmbuf_pool_create("TX_MBUF_POOL", nb_mbufs,
      MEMPOOL_CACHE_SIZE, 0, RTE_MBUF_DEFAULT_BUF_SIZE, SOCKET_ID_ANY);
  if (tx_mbuf_pool == NULL)
    rte_exit(EXIT_FAILURE, "Cannot create tx mbuf pool\n");
  rx_mbuf_pool = rte_pktmbuf_pool_create("RX_MBUF_POOL", nb_mbufs,
      MEMPOOL_CACHE_SIZE, 0, RTE_MBUF_DEFAULT_BUF_SIZE, SOCKET_ID_ANY);
  if (rx_mbuf_pool == NULL)
    rte_exit(EXIT_FAILURE, "Cannot create rx mbuf pool\n");

  if (port_init(0, rx_mbuf_pool) != 0)
    rte_exit(EXIT_FAILURE, "Cannot init port %"PRIu8 "\n", 0);
}
